#ifndef _HMM_H
#define _HMM_H

#include "matrix.h"

typedef struct {
	int n;			/* Number of hidden states */
	int lr;			/* lr>0 means state i can only jump to i+0,1, ... ,lr */
	Vector *pi;		/* Initial state probability vector */
	Matrix *A;		/* State transition probability matrix */
	int v;			/* Number of variables */
	Matrix *B[];	/* One observation probability matrix per variable */
} HMM;

typedef struct {
	int T;			/* Length of each variable sequence */
	int v;			/* Number of variables */
	int *o[];		/* Array of sequence arrays, one for each variable */
} Sequence;

Sequence *sequence_create(int T, int v);
void sequence_destroy(Sequence *seq);
HMM *hmm_create(int n, int v, int m[], int lr);
void hmm_dump(HMM *hmm, FILE *file);
HMM *hmm_load(FILE *file);
void hmm_destroy(HMM *hmm);
void hmm_randomize(HMM *hmm);
HMM *hmm_train(HMM *old, Sequence *seq[], int n);
double hmm_compare(HMM *hmm, Sequence *seq);

#endif

