#ifndef _STROKE_H
#define _STROKE_H

#include "hmm.h"

typedef struct {
	HMM *hmm;			/* HMM describing the stroke */
	int n;				/* Number of samples available */
	int seq_alloc;		/* Number of entries allocated in seq array */
	Sequence *seq[];	/* Array of samples */
} Stroke;

typedef struct {
	int strokes;
	char *str;
	int stroke_nr[];
} Character;

typedef struct {
	int strokes;
	Stroke **stroke;
	int character_alloc;
	int characters;
	Character **character;
} Language;

Stroke *stroke_create(void);
void stroke_destroy(Stroke *stroke);
Stroke *stroke_add_sequence(Stroke *stroke, Sequence *seq);

Character *character_create(Language *language, int strokes, int stroke_nr[],
	char *str);
void character_destroy(Character *character);
double character_compare(Language *language, Character *character, 
	Sequence *seq[], int n);
void character_add_example(Language *language, Character *character,
	Sequence *seq[], int n);

Language *language_create(void);
void language_dump(Language *language, FILE *file);
Language *language_load(FILE *file);
void language_init_strokes(Language *language);
int language_load_strokes(Language *language, FILE *file);
void language_add_character(Language *language, Character *character);
Character *language_select(Language *language, Sequence *seq[], int n);
void language_train(Language *language);

#endif

